function [x11, z11, P11, x1T, z1T, P1T, ll, K, x1S, z1S, z1Sfull, sll] = kalman(F1,H1,Q1,R1, z, Nx, lags, xstart);

Nz = size(z,1);
Nt = size(z,2);
Nx = size(F1,1);

z11 = z;
z1T = z;
z1S = z;
z1Sfull = z;

% Kalman Filter
% Model:
%   x(t) = F(t)*x(t-1) + w(t),  where w(t) ~ N(0,Q(t))
%   z(t) = H(t)*x(t)   + v(t),  where v(t) ~ N(0,R(t))
% Inputs:
%   z  = (Nz x Nt) data matrix
%   dt = (1 x Nt) serial datetime vector 
%        (if scalar or empty assume uniform sampling)
%   F  = (Nx x Nx x Nt) state transition matrices 
%   Q  = (Nx x Nx x Nt) variance of state shocks
%   H  = (Nz x Nx x Nt) observation matrices
%   R  = (Nz x Nz x Nt) variance of observation noise 
% Outputs:
%   x10 = (Nx x Nt) state estimate at t given signal at t-1
%   x11 = (Nx x Nt) state estimate at t given signal at t
%   P10 = (Nx x Nx x Nt) var of state at t given signal at t-1
%   P11 = (Nx x Nx x Nt) var of state at t given signal at t
%   u   = (Nz x Nt) matrix of innovations
%   S   = (Nz x Nt) matrix of innovations
% Misc
%   t   = elapsed time from dt(t-1) to dt(t)
%   k   = (1 x Nz) column index of non-missing variables
% e11   = Standard errors of filtered state, x11
%   K   = Kalman gain

dt = 1:Nt;

% initialize 

x10 = repmat(NaN,[Nx   ,Nt]);
x11 = repmat(NaN,[Nx   ,Nt]);
P10 = repmat(NaN,[Nx,Nx,Nt]);
P11 = repmat(NaN,[Nx,Nx,Nt]);
e11 = repmat(NaN,[Nx   ,Nt]);
K   = repmat(NaN,[Nx,Nz,Nt]);

xstart
x10(:,1)   = zeros(Nx,1);
if all(isfinite(xstart)),
    x10(:,1) = xstart;
end;
x11(:,1)   = x10(:,1);
P10(:,:,1) = eye(Nx);
P11(:,:,1) = eye(Nx);

INx = eye(Nx);

z11 = z;
mi = isnan(z(:,1));
z11(mi,1) = H1(mi,:)*x11(:,1);
S  = [];
uu = [];

for t=2:Nt
    
    % predict state
    x10(:,t)   = F1(:,:)*x11(:,t-1);
    P10(:,:,t) = F1(:,:)*P11(:,:,t-1)*F1(:,:)' + Q1(:,:);
    
    % update 
    k          = isfinite( z(:,t) );
    if max(k)==0;
        x11(:,t)   = x10(:,t);
        P11(:,:,t) = P10(:,:,t);
        ll(1,t) = 0;
    else;
        
        u          = z(k,t) - H1(k,:)*x10(:,t);
        S          = H1(k,:)*(P10(:,:,t)*H1(k,:)') + R1(k,k);
        K(:,k,t)   = (P10(:,:,t)*H1(k,:)')*inv(S);
        x11(:,t)   = x10(:,t) + K(:,k,t)*u;
        P11(:,:,t) = (INx - K(:,k,t)*H1(k,:))*P10(:,:,t);
        ll(1,t)    = -sum(k)/2*log(2*pi) - 1/2*log(max(det(S),1e-20)) - 1/2*u'*inv(S)*u;
    end
    
    % predict observation data;
    mi = isnan(z(:,t));
    z11(mi,t) = H1(mi,:)*x11(:,t);
    
end

%smoothed estimates (x1T, p1T, z1T), Gibbs samples (x1S, z1S), (latter for missing z)

% initialize
x1T = 0*x11;
x1S = 0*x11;
P1T = 0*P11;
z1T = z11;
z1S = z11;
z1Sfull = 0*z11;
szT = 0*z11;
sxT = 0*x11;

% sample-end values
x1T(:,Nt)   = x11(:,Nt);
P1T(:,:,Nt) = P11(:,:,Nt);
mi          = isnan(z(:,Nt)); smi = sum(mi);

 for tt = Nt-1:-1:1,
    %smooth
    Pstar = P11(:,:,tt)*F1(:,:)'*inv(P10(:,:,tt+1));
    x1T(:,tt)     = x11(:,tt) + Pstar*(x1T(:,tt+1)-F1(:,:)*x11(:,tt));
    P1T(:,:,tt)   = P11(:,:,tt) + Pstar*(P1T(:,:,tt+1)-P10(:,:,tt+1))*Pstar';
    sxT(:,tt) = real(sqrt(diag(P1T(:,:,tt))));
    % fill in missings
    z1T(mi,tt) = H1(mi,:)*x1T(:,tt);
 end;

    
pstart = 1;
figure(1);
for j=1:Nx
    subplot(Nx,1,j)
    %z_up = z1T(j,:) + 2*szT(j,:);
    %z_dn = z1T(j,:) - 2*szT(j,:);
    x_up = x1T(j,pstart:end) + 2*sxT(j,pstart:end);
    x_dn = x1T(j,pstart:end) - 2*sxT(j,pstart:end);
    %plot([x1T(j,pstart:end); x1S(j,pstart:end); x_up; x_dn]');
    plot([x1T(j,pstart:end); x_up(1,pstart:end); x_dn(1,pstart:end)]');
    title(['Factor ' num2str(j) ' with 95% Conf Int'])
end

sll = sum(ll);